#include "sll/sll_functions.h"
#include "dll/dll_functions.h"
#include "bst/bst_functions.h"
#include "avl/avl_functions.h"

//	#define _test_sll
//	#define _test_dll
//	#define _test_bst
//	#define _test_avl

	int main(void) {
		#ifdef _test_sll
			/* Create and initialize myHeader */
			sll_header_t myHeader;
				myHeader.links[_sll_headerlink_head] = \
				myHeader.links[_sll_headerlink_tail] = 0x00000000;
			
			/* Create and Initialize a couple test nodes */
			sll_node_t testnode_a, testnode_b, testnode_c;
			testnode_a.nextlink = testnode_b.nextlink = testnode_c.nextlink = 0x00000000;
	
			/* Test insertion */
			sll_insert(&myHeader, &testnode_a);
			sll_insert(&myHeader, &testnode_b);
			sll_insert(&myHeader, &testnode_c);

			/* Check the List */
			sll_showlist(&myHeader);
		
			/* Test removal (Remove all Nodes) */
			sll_remove(&myHeader, &testnode_c);
			sll_remove(&myHeader, &testnode_b);
			sll_remove(&myHeader, &testnode_a);

			/* Check the List (It should be completely zeroed now */
			sll_showlist(&myHeader);
		#endif

		#ifdef _test_dll
			/* Create and initialize myHeader */
			dll_header_t myHeader;
				myHeader.links[_dll_headerlink_head] = \
				myHeader.links[_dll_headerlink_tail] = 0x00000000;

			/* Create and Initialize a couple test nodes */
			dll_node_t testnode_a, testnode_b, testnode_c;
			testnode_a.links[_dll_nodelink_previous] = \
			testnode_b.links[_dll_nodelink_previous] = \
			testnode_c.links[_dll_nodelink_previous] = \
			testnode_a.links[_dll_nodelink_next] = \
			testnode_b.links[_dll_nodelink_next] = \
			testnode_c.links[_dll_nodelink_next] = 0x00000000;

			/* Test insertion */
			dll_insert(&myHeader, &testnode_a);
			dll_insert(&myHeader, &testnode_b);
			dll_insert(&myHeader, &testnode_c);

			/* Check the List */
			dll_showlist(&myHeader);

			/* Test removal (Remove all Nodes) */
			dll_remove(&myHeader, &testnode_b);
			dll_remove(&myHeader, &testnode_a);
			dll_remove(&myHeader, &testnode_c);

			/* Check the List (It should be completely zeroed now */
			dll_showlist(&myHeader);

		#endif

		#ifdef _test_bst
			/* Create and initialize a Tree */
			bst_node_t *testtree = 0;
		
			/* Create and inititalize some test nodes */
			bst_node_t testnode_a, testnode_b, testnode_c, testnode_d, testnode_e;

			/* Test Insertion */
			bst_insert(&testtree, bst_initialize(&testnode_a, 1));
			bst_insert(&testtree, bst_initialize(&testnode_b, 2));
			bst_insert(&testtree, bst_initialize(&testnode_c, 4));
			bst_insert(&testtree, bst_initialize(&testnode_d, 3));
			bst_insert(&testtree, bst_initialize(&testnode_e, 5));

			/* Test location */
			bst_node_t *testnode_ptr = 0;

			/* Display Tree */
			bst_showtree(&testtree, testtree);

			/* Test Node removal. (These calls will return the address
			   of the Node actually being removed. In the case of 
			   Subtree deletion, it will return the address of the 
			   'Replacement' Node (Because the Subtree Node's key
			   is simply being overwritten by that of the Replacement). */
			bst_remove(&testtree, bst_locate(&testtree, 1));
			bst_remove(&testtree, bst_locate(&testtree, 4));
			bst_remove(&testtree, bst_locate(&testtree, 3));
			bst_remove(&testtree, bst_locate(&testtree, 2));
			bst_remove(&testtree, bst_locate(&testtree, 5));

			_macro_dbgout("----\r\n");

			/* Display Tree (No tree should show ... ) */
			bst_showtree(&testtree, testtree);

		#endif

		#ifdef _test_avl
			/* Create nodes on the stack */
			avl_node_t testnode_array[0xF];

			/* Create and initialize our tree */
			avl_node_t *testtree = 0;

			/* Initialize and insert our Nodes */
			for(uint32_t counter = 0; counter < 0xF; counter++) {
				avl_insert(&testtree, avl_initialize(&testnode_array[counter], counter));
			}

			/* Display the tree (Tree should be completely balanced) */
			avl_showtree(&testtree, testtree);

			/* Test Removal */
			for(uint32_t counter = 0; counter < 0xF; counter++) {
				avl_remove(&testtree, testtree);
			}

			_macro_dbgout("----\r\n");

			/* Display the tree (There should be no tree) */
			avl_showtree(&testtree, testtree);
		#endif

		#ifdef __WIN32__
			system("pause");
		#endif
		return _ret_success;
	}
